@@ -1,18 +1,27 @@
 
+from __future__ import print_function, absolute_import
+
 import os
-import cmd
-import types
+import sys
+cmd = sys.modules["pymol.cmd"]
 from pymol import _cmd
 import threading
 import traceback
-import thread
+
+if sys.version_info[0] == 2:
+    import thread
+    import urllib2
+else:
+    import _thread as thread
+    import urllib.request as urllib2
+
 import re
 import time
 import pymol
 
-from chempy import io
+import chempy.io
 
-from cmd import DEFAULT_ERROR, DEFAULT_SUCCESS, loadable, _load2str, Shortcut, \
+from .cmd import DEFAULT_ERROR, DEFAULT_SUCCESS, loadable, _load2str, Shortcut, \
    is_string, is_ok
 
 # cache management:
@@ -63,9 +72,9 @@ def _cache_purge(max_size, _self=cmd):
             if max_size>=0: # purge to reduce size
                 now = time.time()
                 # sort by last access time
-                new_cache = map(lambda x:[(now-x[5])/x[4],x], _pymol._cache)
+                new_cache = [[(now-x[5])/x[4],x] for x in _pymol._cache]
                 new_cache.sort()
-                new_cache = map(lambda x:x[1],new_cache)
+                new_cache = [x[1] for x in new_cache]
                 # remove oldest entries one by one until size requirement is met
                 while (cur_size>max_size) and (len(new_cache)>1):
                     entry = new_cache.pop() 
@@ -257,8 +266,9 @@ def _do(cmmd,log=0,echo=1,_self=cmd):
 # movie rendering
 
 def _mpng(prefix, first=-1, last=-1, preserve=0, modal=0,
-          format=-1, mode=-1, quiet=1, _self=cmd): # INTERNAL
-    import sys
+          format=-1, mode=-1, quiet=1,
+          width=0, height=0,
+          _self=cmd): # INTERNAL
     format = int(format)
     # WARNING: internal routine, subject to change
     try:
@@ -275,7 +285,8 @@ def _mpng(prefix, first=-1, last=-1, preserve=0, modal=0,
         fname = cmd.exp_path(fname)
         r = _cmd.mpng_(_self._COb,str(fname),int(first),
                        int(last),int(preserve),int(modal),
-                       format,int(mode),int(quiet))
+                       format,int(mode),int(quiet),
+                       int(width), int(height))
     finally:
         _self.unlock(-1,_self)
     return r
@@ -302,10 +313,9 @@ def file_read(finfo, _self=cmd):
     finfo may be a filename, URL or open file handle.
     '''
     try:
-        if not isinstance(finfo, basestring):
+        if not is_string(finfo):
             handle = finfo
         elif '://' in finfo:
-            import urllib2
             req = urllib2.Request(finfo,
                     headers={'User-Agent': 'PyMOL/' + _self.get_version()[0]})
             handle = urllib2.urlopen(req)
@@ -316,12 +326,12 @@ def file_read(finfo, _self=cmd):
     except IOError:
         raise pymol.CmdException('failed to open file "%s"' % finfo)
 
-    if contents[:2] == '\x1f\x8b': # gzip magic number
-        import cStringIO, gzip
-        fakestream = cStringIO.StringIO(contents)
+    if contents[:2] == b'\x1f\x8b': # gzip magic number
+        import io, gzip
+        fakestream = io.BytesIO(contents)
         return gzip.GzipFile(fileobj=fakestream).read()
 
-    if contents[:2] == 'BZ' and contents[4:10] == '1AY&SY': # bzip magic
+    if contents[:2] == b'BZ' and contents[4:10] == b'1AY&SY': # bzip magic
         import bz2
         return bz2.decompress(contents)
 
@@ -339,26 +349,27 @@ def download_chem_comp(resn, quiet=1, _self=cmd):
         return filename
 
     url = "ftp://ftp.ebi.ac.uk/pub/databases/msd/pdbechem/files/mmcif/" + resn + ".cif"
+    url = "http://files.rcsb.org/ligands/download/" + resn + ".cif"
     if not quiet:
-        print ' Downloading', url
+        print(' Downloading ' + url)
 
     try:
         contents = _self.file_read(url)
         if not contents: raise
     except:
-        print ' Error: Download failed'
+        print(' Error: Download failed')
         return ''
 
     try:
-        with open(filename, 'w') as handle:
+        with open(filename, 'wb') as handle:
             handle.write(contents)
     except IOError as e:
-        print e
-        print 'Your "fetch_path" setting might point to a read-only directory'
+        print(e)
+        print('Your "fetch_path" setting might point to a read-only directory')
         return ''
 
     if not quiet:
-        print '  ->', filename
+        print('  ->' + filename)
 
     return filename
 
@@ -373,66 +384,18 @@ def _load(oname,finfo,state,ftype,finish,discrete,
     r = DEFAULT_ERROR
     size = 0
     if ftype not in (loadable.model,loadable.brick):
-        if ftype == loadable.r3d:
-            import cgo
-            obj = cgo.from_r3d(finfo)
-            if is_ok(obj):
-                r = _cmd.load_object(_self._COb,str(oname),obj,int(state)-1,loadable.cgo,
-                                      int(finish),int(discrete),int(quiet),
-                                      int(zoom))
-                _self.set("two_sided_lighting", 0, str(oname))
-            else:
-                print "Load-Error: Unable to open file '%s'."%finfo
-        elif ftype == loadable.cc1: # ChemDraw 3D
-            obj = io.cc1.fromFile(finfo)
-            if obj:
-                r = _cmd.load_object(_self._COb,str(oname),obj,int(state)-1,loadable.model,
-                                      int(finish),int(discrete),
-                                      int(quiet),int(zoom))
-        elif ftype == loadable.moe:
-            try:
-                # BEGIN PROPRIETARY CODE SEGMENT
-                from epymol import moe
-                moe_str = _self.file_read(finfo)
-                r = moe.read_moestr(moe_str,str(oname),int(state),
-                                int(finish),int(discrete),int(quiet),int(zoom),_self=_self)
-
-                # END PROPRIETARY CODE SEGMENT
-            except ImportError:
-                print "Error: .MOE format not supported by this PyMOL build."
-                if _self._raising(-1,_self): raise pymol.CmdException
-
-        elif ftype == loadable.mae:
-            try:
-                # BEGIN PROPRIETARY CODE SEGMENT
-                from epymol import mae
-                mae_str = _self.file_read(finfo)
-                r = mae.read_maestr(mae_str,str(oname),
-                                    int(state),
-                                    int(finish),int(discrete),
-                                    int(quiet),int(zoom),int(multiplex),
-                                    int(mimic),
-                                    _self=_self)
-
-                # END PROPRIETARY CODE SEGMENT
-            except ValueError:
-                print "Error: .MAE format not supported by this PyMOL build."
-                if _self._raising(-1,_self): raise pymol.CmdException
-
-        else:
-            if ftype in _load2str and ('://' in finfo or cmd.gz_ext_re.search(finfo)):
-                # NOTE: we could safely always do this, not only for URLs and
-                # compressed files. But I don't want to change the old behavior
-                # that regular files are read from the C function.
+        if True:
+            if ftype in _load2str:
                 finfo = _self.file_read(finfo)
                 ftype = _load2str[ftype]
             r = _cmd.load(_self._COb,str(oname),finfo,int(state)-1,int(ftype),
                           int(finish),int(discrete),int(quiet),
-                          int(multiplex),int(zoom), plugin, object_props)
+                          int(multiplex),int(zoom), plugin,
+                          object_props, atom_props, int(mimic))
     else:
         try:
-            x = io.pkl.fromFile(finfo)
-            if isinstance(x,types.ListType) or isinstance(x,types.TupleType):
+            x = chempy.io.pkl.fromFile(finfo)
+            if isinstance(x, (list, tuple)):
                 for a in x:
                     r = _cmd.load_object(_self._COb,str(oname),a,int(state)-1,
                                                 int(ftype),0,int(discrete),int(quiet),
@@ -447,79 +410,133 @@ def _load(oname,finfo,state,ftype,finish,discrete,
                                             int(quiet),int(zoom))
         except:
 #            traceback.print_exc()
-            print "Load-Error: Unable to load file '%s'." % finfo
+            print("Load-Error: Unable to load file '%s'." % finfo)
     return r
 
 # function keys and other specials
 
+modifier_keys = [
+    '',
+    'SHFT',
+    'CTRL',
+    'CTSH',
+    'ALT',
+]
+
+special_key_codes = {
+    # GLUT special key codes (see glutSpecialFunc)
+
+    1        :  'F1',
+    2        :  'F2',
+    3        :  'F3',
+    4        :  'F4',
+    5        :  'F5',
+    6        :  'F6',
+    7        :  'F7',
+    8        :  'F8',
+    9        :  'F9',
+    10       :  'F10',
+    11       :  'F11',
+    12       :  'F12',
+
+    100      :  'left',
+    101      :  'up',
+    102      :  'right',
+    103      :  'down',
+    104      :  'pgup',
+    105      :  'pgdn',
+    106      :  'home',
+    107      :  'end',
+    108      :  'insert',
+}
+
+special_key_names = set(special_key_codes.values())
+
+def _invoke_key(key, quiet=0, _self=cmd):
+    '''Invoke a function that was mapped with cmd.set_key()'''
+    try:
+        mapping = _self.key_mappings[key]
+    except KeyError:
+        mapping = None
+
+    if not mapping:
+        if not quiet:
+            print(" No key mapping for '%s'" % (key))
+        return False
+
+    if is_string(mapping):
+        _self.do(mapping)
+    else:
+        fn, args, kwargs = mapping
+        fn(*args, **kwargs)
+
+    return True
+
 def _special(k,x,y,m=0,_self=cmd): # INTERNAL (invoked when special key is pressed)
     pymol=_self._pymol
     # WARNING: internal routine, subject to change
     k=int(k)
     m=int(m)
-    my_special = _self.special
-    if(m>0) and (m<5):
-        my_special = (_self.special,
-                      _self.shft_special,
-                      _self.ctrl_special,
-                      _self.ctsh_special,
-                      _self.alt_special)[m]
-    if my_special.has_key(k):
-        if my_special[k][1]:
-            apply(my_special[k][1],my_special[k][2],my_special[k][3])
-        else:
-            key = my_special[k][0]
-            if(m>0) and (m<5):
-                key = ('','SHFT-','CTRL-','CTSH-','ALT-')[m] + key
-            if key in _self.get_scene_list():
-                _self.scene(key)
-            elif is_string(pymol._scene_dict_sc.interpret(key+"-")):
-                _self.scene(pymol._scene_dict_sc[key+"-"])
-            elif pymol._view_dict.has_key(key):
-                _self.view(key)
-            elif is_string(pymol._view_dict_sc.interpret(key+"-")):
-                _self.view(pymol._view_dict_sc[key+"-"])
-    return None
+
+    # convert numeric codes to string key
+
+    try:
+        key = special_key_codes[k]
+
+        if m:
+            key = modifier_keys[m] + '-' + key
+    except KeyError:
+        return False
+
+    # check for explicit mapping
+
+    if _invoke_key(key, 1, _self):
+        return True
+
+    # check for scenes and views
+
+    for (fn, sc) in [
+            (_self.scene, pymol._scene_dict_sc),
+            (_self.view,  pymol._view_dict_sc),
+            ]:
+        if key in sc.keywords:
+            fn(key)
+            return True
+
+        autocomp = sc.interpret(key + '-')
+        if is_string(autocomp):
+            fn(autocomp)
+            return True
+
+    print(" No key mapping and no scene or view for '%s'" % (key))
+    return False
 
 # control keys
 
 def _ctrl(k,_self=cmd):
     # WARNING: internal routine, subject to change
-    if _self.ctrl.has_key(k):
-        ck = _self.ctrl[k]
-        if ck[0]!=None:
-            apply(ck[0],ck[1],ck[2])
-    return None
+    _invoke_key('CTRL-' + k, 0, _self)
 
 # alt keys
 
 def _alt(k,_self=cmd):
     # WARNING: internal routine, subject to change
-    if _self.alt.has_key(k):
-        ak = _self.alt[k]
-        if ak[0]!=None:
-            apply(ak[0],ak[1],ak[2])
-    return None
+    _invoke_key('ALT-' + k.upper(), 0, _self)
 
 # command (apple) keys
 
 def _cmmd(k,_self=cmd):
     # WARNING: internal routine, subject to change
     # command-key on macs
-    if _self.cmmd.has_key(k):
+    if k in _self.cmmd:
         ak = _self.cmmd[k]
         if ak[0]!=None:
-            apply(ak[0],ak[1],ak[2])
+            ak[0](*ak[1], **ak[2])
     return None
 
 def _ctsh(k,_self=cmd):
     # WARNING: internal routine, subject to change
-    # command-key on macs
-    if _self.ctsh.has_key(k):
-        ak = _self.ctsh[k]
-        if ak[0]!=None:
-            apply(ak[0],ak[1],ak[2])
-    return None
+    _invoke_key('CTSH-' + k, 0, _self)
     
 
 # writing PNG files (thread-unsafe)
@@ -596,7 +613,6 @@ def _refresh(swap_buffers=1,_self=cmd):  # Only call with GLUT thread!
 # stereo (platform dependent )
 
 def _sgi_stereo(flag): # SGI-SPECIFIC - bad bad bad
-    import sys
     # WARNING: internal routine, subject to change
     if sys.platform[0:4]=='irix':
         if os.path.exists("/usr/gfx/setmon"):
@@ -676,25 +692,25 @@ def _dump_floats(lst,format="%7.3f",cnt=9):
     # WARNING: internal routine, subject to change
     c = cnt
     for a in lst:
-        print format%a,
+        print(format%a, end=' ')
         c = c -1
         if c<=0:
-            print
+            print()
             c=cnt
     if c!=cnt:
-        print
+        print()
 
 def _dump_ufloats(lst,format="%7.3f",cnt=9):
     # WARNING: internal routine, subject to change
     c = cnt
     for a in lst:
-        print format%abs(a),
+        print(format%abs(a), end=' ')
         c = c -1
         if c<=0:
-            print
+            print()
             c=cnt
     if c!=cnt:
-        print
+        print()
 
 # HUH?
 def _adjust_coord(a,i,x):
