@@ -16,9 +16,10 @@ from __future__ import print_function, absolute_import
 
 if __name__=='pymol.viewing':
 
-    try:
+    import sys
+    if sys.version_info[0] == 2:
         import thread
-    except ImportError:
+    else:
         import _thread as thread
 
     import threading
@@ -29,7 +30,7 @@ if __name__=='pymol.viewing':
     import copy
     from . import parsing
     import re
-    cmd = __import__("sys").modules["pymol.cmd"]
+    cmd = sys.modules["pymol.cmd"]
     
     from .cmd import _cmd,lock,unlock,Shortcut,QuietException,_raising, \
           _feedback,fb_module,fb_mask, \
@@ -803,6 +804,7 @@ SEE ALSO
         finally:
             _self.unlock(r,_self)
         if is_ok(r):
+            output = int(output)
             if len(r):
                 if (_self.get_setting_int("logging") != 0) and (output<3):
                     if not quiet:
@@ -1170,7 +1172,7 @@ SEE ALSO
 
     def scene(key='auto', action='recall', message=None, view=1,
               color=1, active=1, rep=1, frame=1, animate=-1,
-              new_key=None, hand=1, quiet=1, _self=cmd):
+              new_key=None, hand=1, quiet=1, sele="all", _self=cmd):
         '''
 DESCRIPTION
 
@@ -1249,7 +1251,7 @@ SEE ALSO
         with _self.lockcm:
             r = _cmd.scene(_self._COb, key, action, message, int(view),
                     int(color), int(active), int(rep), int(frame),
-                    float(animate), new_key, int(hand))
+                    float(animate), new_key, int(hand), sele)
 
         # autocomplete
         if action in ('store', 'delete') or action.startswith('insert_'):
@@ -1260,6 +1262,66 @@ SEE ALSO
 
         return r
                         
+    def _legacy_scene(key='auto', action='recall', message=None, view=1,
+              color=1, active=1, rep=1, frame=1, animate=-1,
+              new_key=None, hand=1, quiet=1, _self=cmd):
+        ''' FOR INTERNAL USE ONLY. Stores and deletes <=1.7.4 compatible scenes. '''
+        pymol=_self._pymol
+        r = DEFAULT_SUCCESS
+        view = int(view)
+        rep = int(rep)
+        color = int(color)
+        active = int(active)
+        frame = int(frame)
+        quiet = int(quiet)
+        animate = 0
+
+        try:
+            _self.lock(_self) # manipulating global data, so need lock
+            if key=='*':
+                if action=='clear':
+                    for key in pymol._scene_dict:
+                        # free selections
+                        scene_list = pymol._scene_dict[key]
+                        if len(scene_list)>3:
+                            colorection = scene_list[3]
+                            if colorection!=None:
+                                _self.del_colorection(colorection,key)
+                        name = "_scene_"+key+"_*"
+                        _self.delete(name)
+                else:
+                    raise ValueError('action=' + action)
+            else:
+                if action == 'store':
+                    if key in ('new', 'auto'):
+                        raise ValueError('key=' + key)
+                    if key in pymol._scene_dict:
+                        raise RuntimeError('update not supported')
+                    if rep:
+                        for rep_name in rep_list:
+                            name = "_scene_"+key+"_"+rep_name
+                            _self.select(name,"rep "+rep_name)
+                    if is_string(message):
+                        if message:
+                            if (message[0:1] in [ '"',"'"] and
+                                 message[-1:] in [ '"',"'"]):
+                                message=message[1:-1]
+                            else:
+                                message = message.splitlines()
+                    pymol._scene_dict[key] = [
+                        _self.get_view(0) if view else None,
+                        _self.get_vis() if active else None,
+                        _self.get_frame() if frame else None,
+                        _self.get_colorection(key) if color else None,
+                        1 if rep else None,
+                        message,
+                    ]
+                else:
+                    raise ValueError('action=' + action)
+        finally:
+            _self.unlock(r,_self)
+        return r
+
     def session_save_views(session,_self=cmd):
         pymol=_self._pymol        
         session['view_dict']=copy.deepcopy(pymol._view_dict)
@@ -1602,17 +1664,9 @@ PYMOL API
         '''
         r = None
         if cmd.is_string(width) and height == -1:
-            try:
-                width = eval(re.sub(r"[^0-9,\-\)\(\.]","",width))
-            except:
-                traceback.print_exc()
-                print("Error: bad viewport argument; should be 2 floats, width and height.")
-                raise QuietException
-            if len(width)!=2:
-                print("Error: bad viewport argument; should be 2 floats, width and height.")
-                raise QuietException
-            height = width[1]
-            width = width[0]
+            width = _self.safe_eval(width)
+            if _self.is_sequence(width):
+                width, height = width
 
         if not cmd.is_glut_thread():
             _self.do("viewport %d,%d"%(int(width),int(height)),0)
@@ -2287,7 +2341,7 @@ PYMOL API
         if palette_hit:
             palette = palette_hit
 
-        if expression not in ('count', 'b', 'q', 'pc') or not palette_hit:
+        if not expression.replace('_', '').isalpha() or not palette_hit:
             return spectrumany(expression, palette, selection,
                     minimum, maximum, quiet, _self)
         
